/* fopenx.c: this opens a file with a suggested extension.  It allows the
 *  *  input filename to be missing the provided extension.
 *   * Copyright  July 23, 1999: Charles E. Campbell, Jr. -- see <Copyright> file
 *    *
 *     *  Routines Include: <fopenx.c>
 *      *    FILE *fopenx( char *filename,  char *mode,  char *ext,  char *env_var)
 *       *    void fopenx_fileput(char *fmt,...)
 *        */
#include <stdio.h>
#include "xtdio.h"

/* ----------------------------------------------------------------------
 *  * Global Data
 *   */
char *fopenxfile     = NULL;

/* ---------------------------------------------------------------------- */

/* fopenx: fopen with automatic extension handling and PATH-ing
 *  *
 *   *  Returns: success = a valid FILE pointer
 *    *           failure = a null FILE pointer
 *     *                     fopen[xv]file will be freed and nulled on failure
 *      *
 *       *  Side Effects:
 *        *   fopenxfile will point to any successfully opened file.
 *         *   fopenvfile will point to the full pathname to any successfully
 *          *              opened file assuming fopenv was used.  If fopenx opened
 *           *              a file in the current directory, fopenxfile will be
 *            *              non-null but fopenvfile will be null.
 *             */
FILE *fopenx(
		  char *filename,	/* name of file to be opened, may be missing suffix		*/
		    char *mode,		/* r for reading, w for writing							*/
		      char *ext,		/* extension/suffix: may or may not have leading .		*/
		        char *env_var)	/* environment variable for PATH-like searching			*/
{
	char *dirsep= NULL;
	char *fn_ext= NULL;
	FILE *fp    = NULL;


	/* free up old fopenvfile storage */
	if(fopenvfile) {
			free(fopenvfile);
				fopenvfile= NULL;
					}

	/* sanity checks */
	if(!filename) {
			error(XTDIO_WARNING,"fopenx: null filename\n");
				return fp;
					}

	if(!mode) {
			error(XTDIO_WARNING,"fopenx: null mode\n");
				return fp;
					}

	if(!ext) {	/* no user-supplied extension, use fopenv */
			fp= fopenv(filename,mode,env_var);
				if(fp) {
							fopenx_fileput(fopenvfile);
									fopenvfile= NULL;			/* prevent fopenx from freeing fopenvfile */
											}
					return fp;
						}

	/* ignore leading . in ext if any */
	if(ext[0] == '.') ++ext;

#ifdef vms
	/* locate extension (if any) */
	dirsep= strchr(filename,']');
	if(!dirsep) dirsep= strrchr(filename,':');
	fn_ext= strrchr(filename,'.');
	if(fn_ext < dirsep) fn_ext= NULL;
#endif

#ifdef unix
	dirsep= strrchr(filename,'/');
	fn_ext= strrchr(filename,'.');
	if(fn_ext < dirsep) fn_ext= NULL;
#endif

#ifdef msdos
	dirsep= (char *) strrchr(filename,'\\');
	fn_ext= (char *) strrchr(filename,'.');
	if(fn_ext < dirsep) fn_ext= NULL;
#endif

#ifdef AZTEC_C
	dirsep= strrchr(filename,'/');
	fn_ext= strrchr(filename,'.');
	if(fn_ext < dirsep) fn_ext= NULL;
#endif

	/* get fn_ext to point just past '.', too */
	if(fn_ext) ++fn_ext;

	/* filename has extension already provided, and it matches */
	if(fn_ext && !strcmp(fn_ext,ext)) {

			/* use environment variable */
			if(env_var && env_var[0]) {
						fp= fopenv(filename,mode,env_var);
								}

				/* look only in current directory */
				else {
							fp= fopen(filename,mode);
									if(fp) fopenx_fileput(filename);
											}

					return fp;
						}

#ifdef vms
	/* append suggested extension to filename (removing current extension if any) */
	if(fn_ext) fopenx_fileput("%s%s", filename,fn_ext);
	else       fopenx_fileput("%s.%s",filename,ext);
#else
	/* append suggested extension to filename */
	fopenx_fileput("%s.%s",filename,ext);
#endif

	/* open the file and use environment variable */
	if(env_var && env_var[0]) {
			fp= fopenv(fopenxfile,mode,env_var);
				}

	/* open file but look only in current directory */
	else {
			fp= fopen(fopenxfile,mode);
				}

	return fp;
}

/* --------------------------------------------------------------------- */

/* fopenx_fileput: put a filename into the fopenxfile.  Note that the
 *  *   fopenxfile buffer constantly grows
 *   */
void fopenx_fileput(char *fmt,...)
{
	va_list       args;
	static size_t fopenxfile_len = 0;
	int           fmtlen;

	/* sanity check -- null fopenxfile means the length had better be zero */
	if(!fopenxfile) fopenxfile_len= 0;

	/* allocate or re-allocate fopenxfile as needed */
	fmtlen= strlen(fmt);
	if(fmtlen > fopenxfile_len) {
			fopenxfile_len= fmtlen + 256;
				if(!fopenxfile) fopenxfile= (char *) calloc(fopenxfile_len,sizeof(char));
					else            fopenxfile= (char *) realloc(fopenxfile,fopenxfile_len*sizeof(char));
						outofmem(fopenxfile,"fopenx_fileput: unable to allocate buffer for <%s>\n",sprt(fmt));
							}

	/* put formatted info into fopenxfile buffer */
	va_start(args,fmt);
	vsprintf(fopenxfile,fmt,args);
	va_end(args);
}

/* --------------------------------------------------------------------- */


/* --------------------------------------------------------------------- */


